@@ -1,9 +1,14 @@
-#!/usr/bin/env python
 # coding: utf-8
 #
 # Copyright (C) 2004-2005 Ross Burton <ross@burtonini.com>
-#               2005-2006 Canonical
+#               2005-2007 Canonical
 #               2006 Sebastian Heinlein
+# Authors:
+#  Ross Burton
+#  Michael Vogt
+#  Sebastian Heinlein
+#  Ian Jackson
+#  Niran Babalola
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -31,6 +36,7 @@ import pango
 
 import gettext
 from gettext import gettext as _
+from math import log
 
 #setup gettext
 app="gnome-app-install"
@@ -39,7 +45,7 @@ gettext.bindtextdomain(app)
 gtk.glade.textdomain(app)
 gtk.glade.bindtextdomain(app)
 
-
+import gc
 import stat
 import glob
 import re
@@ -53,6 +59,7 @@ from datetime import datetime
 import dbus
 import dbus.service
 import dbus.glib
+import time
 
 from warnings import warn
 warnings.filterwarnings("ignore", "ICON:.*", UserWarning)
@@ -61,7 +68,7 @@ import apt
 import apt_pkg
 
 # from update-manager, needs to be factored out
-from UpdateManager.Common.aptsources import SourcesList, is_mirror
+from aptsources.sourceslist import SourcesList, is_mirror
 
 # internal imports
 from DialogNewlyInstalled import DialogNewlyInstalled
@@ -76,8 +83,9 @@ from Menu import ApplicationMenu
 #        the version in g-a-i is a little bit ahead :)
 from ReleaseNotesViewer import ReleaseNotesViewer
 from SimpleGladeApp import SimpleGladeApp
-from Progress import GtkOpProgressWindow
+from Progress import GtkOpProgressWindow, GtkCdromProgress
 from Util import *
+from Cache import MyCache
 import common
 
 
@@ -112,11 +120,12 @@ class AppInstallDbusControler(dbus.service.Object):
     
 class AppInstall(SimpleGladeApp):
 
-    def __init__(self, datadir, desktopdir, cachedir, arguments=None, 
-                 mime_search=None):
+    def __init__(self, datadir, desktopdir, cachedir, arguments, 
+                 activation_style, transient_for_xid=None, addon_cd=None):
         self.setupDbus()
 
         self.search_timeout_id = 0
+	self.activation_style = activation_style
 
         # setup a default icon
         self.icons = common.ToughIconTheme()
@@ -130,6 +139,19 @@ class AppInstall(SimpleGladeApp):
         self.cachedir = cachedir
         self.desktopdir = desktopdir
 
+        # FIXME: use subprocess instead
+        pipe = os.popen("lsb_release -c -s")
+        self.distro = pipe.read().strip()
+        del pipe
+
+        # the GdkWindow of the transient parent
+        self.transient_for = None
+	if transient_for_xid is not None:
+	    self.transient_for = gtk.gdk.window_foreign_new(transient_for_xid)
+            if self.transient_for:
+                self.window_main.realize()
+                self.window_main.window.set_transient_for(self.transient_for)
+
         # sensitive stuff
         self.button_apply.set_sensitive(False)
 
@@ -147,9 +169,14 @@ class AppInstall(SimpleGladeApp):
         self.tooltips = gtk.Tooltips()
         self.tipmap = {}
 
+        # seen flags
+        # FIXME: make those @properties that are auto-syned with gconf
+        self.seen_universe = self.config.get_int("/apps/gnome-app-install/universe_warning")
+        self.seen_multiverse = self.config.get_int("/apps/gnome-app-install/multiverse_warning")
+
         # combobx with filters for the application list
         filter_to_restore = self.config.get_int("/apps/gnome-app-install/filter_applications")
-        if filter_to_restore not in range(5):
+        if filter_to_restore not in range(7):
             filter_to_restore = 0
         list_filters = gtk.ListStore(gobject.TYPE_STRING,
                                      gobject.TYPE_BOOLEAN,
@@ -217,7 +244,7 @@ class AppInstall(SimpleGladeApp):
         self.treeview_packages.set_model(fake_applications)
 
         # now show the main window ...
-        if mime_search:
+        if activation_style.isSpecific():
             self.label_progress.set_markup("<big><b>%s</b></big>\n\n%s" %
                                          (_("Searching for appropriate "
                                             "applications"),
@@ -226,6 +253,11 @@ class AppInstall(SimpleGladeApp):
         else:
             self.window_main.show()
 
+        # check for addon cd and move desktopdir to it
+        # FIXME: test if the CD is for the right architecture/release
+        self.addon_cd = addon_cd
+        self.addCD()
+
         # ... and open the cache
         self.updateCache(filter_to_restore)
 
@@ -240,21 +272,62 @@ class AppInstall(SimpleGladeApp):
         self.multiple_pkgs_seen = set()
 
         # handle arguments
-	if mime_search:
+	if self.activation_style.isSpecific():
             self.window_main.show()
-            self.menu.mimeSearch = mime_search
 	    self.scrolledwindow_left.hide()
-            self.search_entry.set_text(mime_search.string)
+            self.search_entry.set_text(self.activation_style.searchString())
             self.on_search_timeout()
                     
         # create a worker that does the actual installing etc
-        self.packageWorker = PackageWorker()
+        self.packageWorker = PackageWorker(addon_cd)
 
         # used for restore_state
         self.last_toggle = None
 
         # now check if the cache is up-to-date
-        time_cache = 0
+        self._checkAptCache()
+
+    def addCD(self):
+        """ check for addon cd and if available and not yet added
+            add itadd addon cd
+        """
+        if self.addon_cd is not None:
+            try:
+                self.window_main.set_sensitive(False)
+                self.setBusy(True)
+
+                cd_desktopdir = os.path.join(self.addon_cd,"app-install")
+                if os.path.exists(cd_desktopdir):
+                    cdrom = apt.cdrom.Cdrom(progress=GtkCdromProgress(self), mountpoint=self.addon_cd)
+                    if not cdrom.inSourcesList:
+                        cdrom.add()
+                    self.desktopdir=cd_desktopdir
+                    self.cachedir=None
+            except SystemError, e:
+                print "SystemError while adding addon CD:\n%s" % e
+                header = _("Error reading the addon CD")
+                msg = _("The addon CD maybe corrupt ")
+                d = gtk.MessageDialog(parent=self.window_main,
+                                      flags=gtk.DIALOG_MODAL,
+                                      type=gtk.MESSAGE_ERROR,
+                                      buttons=gtk.BUTTONS_CLOSE)
+                d.set_title("")
+                d.set_markup("<big><b>%s</b></big>\n\n%s" % (header, msg))
+                d.realize()
+                d.window.set_functions(gtk.gdk.FUNC_MOVE)
+                d.run()
+                d.destroy()
+                sys.exit(1)
+
+
+
+    def _checkAptCache(self):
+        # we init with the last time the cache dialog was updated to protect
+        # against:
+        # problem: what can happen is that the sources.list is modified
+        #          but we only get I-M-S hits and the mtime of the Packages
+        #          files do not change
+        time_cache = self.config.get_int("/apps/gnome-app-install/cache_dialog_time")
         for f in glob.glob("/var/lib/apt/lists/*Packages"):
             mt = os.stat(f)[stat.ST_MTIME]
             ct = os.stat(f)[stat.ST_CTIME] 
@@ -262,6 +335,8 @@ class AppInstall(SimpleGladeApp):
                time_cache = mt
             if ct > time_cache:
                time_cache = ct
+        if not os.path.exists("/etc/apt/sources.list"):
+            return 
         time_source = os.stat("/etc/apt/sources.list")[stat.ST_MTIME]
         for f in glob.glob("/etc/apt/sources.list.d/*.list"):
             mt = os.stat(f)[stat.ST_MTIME]
@@ -270,9 +345,6 @@ class AppInstall(SimpleGladeApp):
                 time_source = mt
             if ct > time_source:
                 time_source = ct
-        # FIXME: problem: what can happen is that the sources.list is modified
-        #        but we only get I-M-S hits and the mtime of the Packages
-        #        files do not change
         #print "cache:  ", time_cache
         #print "source: ", time_source
         if time_cache < time_source:
@@ -283,6 +355,7 @@ class AppInstall(SimpleGladeApp):
             self.dialog_cache_outdated.hide()
             if res == gtk.RESPONSE_YES:
                 self.reloadSources()
+                self.config.set_int("/apps/gnome-app-install/cache_dialog_time", int(time.time()))
 
     def separator_filter(self, model, iter, user_data=None):
         """Used to draw a spearator in the combobox for the filters"""
@@ -297,16 +370,20 @@ class AppInstall(SimpleGladeApp):
         except:
             print "warning: could not initiate dbus"
             return
-        proxy_obj = bus.get_object('org.freedesktop.AppInstall', '/org/freedesktop/AppInstallObject')
-        iface = dbus.Interface(proxy_obj, 'org.freedesktop.AppInstallIFace')
         try:
+            proxy_obj = bus.get_object('org.freedesktop.AppInstall', '/org/freedesktop/AppInstallObject')
+            iface = dbus.Interface(proxy_obj, 'org.freedesktop.AppInstallIFace')
             iface.bringToFront()
             #print "send bringToFront"
             sys.exit(0)
         except dbus.DBusException, e:
-            print "no listening object (%s) "% e
-            bus_name = dbus.service.BusName('org.freedesktop.AppInstall',bus)
-            self.dbusControler = AppInstallDbusControler(self, bus_name)
+            #print "no listening object (%s) "% e
+            try:
+                bus_name = dbus.service.BusName('org.freedesktop.AppInstall',bus)
+                self.dbusControler = AppInstallDbusControler(self, bus_name)
+            except Exception, e:
+                print "can't init dbus"
+                return
 
     def setBusy(self, flag):
         """ Show a watch cursor if the app is busy for more than 0.3 sec.
@@ -320,7 +397,6 @@ class AppInstall(SimpleGladeApp):
         while gtk.events_pending():
             gtk.main_iteration()
 
-
     def on_combobox_filter_changed(self, combobox):
         """The filter for the application list was changed"""
         self.setBusy(True)
@@ -373,102 +449,244 @@ class AppInstall(SimpleGladeApp):
         item_text = model.get_value(iter, 0)
         item_disclaimer = model.get_value(iter, 2)
         cell_renderer.set_property('text', item_text)
+        # see LP#87727, no idea why we are sometimes called with
+        # this kind of cell_view argument
+        if isinstance(cell_view, gtk.TreeViewColumn):
+            return
         cell_parent = cell_view.get_parent()
-
-        if isinstance(cell_parent, gtk.MenuItem) \
-           and (cell_parent not in self.tipmap \
-                or self.tipmap[cell_parent] != item_disclaimer):
+        if (isinstance(cell_parent, gtk.MenuItem) 
+            and (cell_parent not in self.tipmap 
+                 or self.tipmap[cell_parent] != item_disclaimer)):
             self.tipmap[cell_parent] = item_disclaimer
             self.tooltips.set_tip(cell_parent, item_disclaimer)
 
-    # install toggle on the treeview
-    def on_install_toggle(self, renderer, path):
-        #print "on_install_toggle: %s %s" % (renderer, path)
-        model = self.treeview_packages.get_model()
-        (name, item, popcon) = model[path]
-        # first check if the operation is save
+    def canNotInstallApp(self, pkg):
+        """ helper that displays a message if the package can not
+            be installed """
+        d = gtk.MessageDialog(parent=self.window_main,
+                              flags=gtk.DIALOG_MODAL,
+                              type=gtk.MESSAGE_ERROR,
+                              buttons=gtk.BUTTONS_CLOSE)
+        d.set_title("")
+        d.set_markup("<big><b>%s</b></big>\n\n%s" % (
+            (_("Cannot install '%s'") % pkg),
+            (_("This application conflicts with other "
+               "installed software. To install '%s' "
+               "the conflicting software "
+               "must be removed first.\n\n"
+               "Switch to the 'synaptic' package manager to resolve this "
+               "conflict.") % pkg)))
+        d.realize()
+        d.window.set_functions(gtk.gdk.FUNC_MOVE)
+        d.run()
+        d.destroy()
+        # reset the cache
+        # FIXME: a "pkgSimulateInstall,remove"  thing would
+        # be nice
+        self.cache.clean()
+        return False
+
+    def _ensureInArchive(self, item):
+        """ check and make sure the package is in available archive """
+        # FIXME: make sure to show a dialog about support status of
+        #        universe/multiverse once, even when universe/mutliverse
+        #        are already enabled
+        #self._ensureInArchive(item)
         pkg = item.pkgname
-        if not (self.cache.has_key(pkg) and \
-           self.cache[pkg].candidateDownloadable):
+        if not (self.cache.has_key(pkg) and 
+                self.cache[pkg].candidateDownloadable):
             for it in self.cache._cache.FileList:
-                if it.Component != "" and it.Component == item.component:
+                if (it.Component != "" and it.Component == item.component and
+                    it.Archive != "" and it.Archive == self.distro):
                     # FIXME: use aptsources to check if no internet sources
                     #        and only cdrom sources are used. if yes continue
                     #        and if not report an error and provide the 
                     #        possibility to apt-get update
                     self.error_not_available(item)
                     return False
+            # check if we have the package in the cache
             self.saveState()
-            if self.addChannel(item):
-                self.last_toggle = name
-                self.restoreState()
-            return
-        if self.cache[pkg].isInstalled:
-            # check if it can be removed savly
-            # FIXME: Provide a list of the corresponding packages or
-            #        apps
-            self.cache[pkg].markDelete(autoFix=False)
-            if self.cache._depcache.BrokenCount > 0:
-                d = gtk.MessageDialog(parent=self.window_main,
-                                      flags=gtk.DIALOG_MODAL,
-                                      type=gtk.MESSAGE_ERROR,
-                                      buttons=gtk.BUTTONS_CLOSE)
-                d.set_title("")
-                d.set_markup("<big><b>%s</b></big>\n\n%s" % \
-                             ((_("Cannot remove '%s'") % pkg),
-                              (_("One or more applications depend on %s. "
-                                "To remove %s and the dependent applications, "
-                                "use the Synaptic package manager.") % (pkg, 
-                                                                        pkg))))
-                d.realize()
-                d.window.set_functions(gtk.gdk.FUNC_MOVE)
-                d.run()
-                d.destroy()
-                self.cache.clean()
-                return
-            self.cache[pkg].markKeep()
-            # FIXME: those assert may be a bit too strong,
-            # we may just rebuild the cache if something is
-            # wrong
-            assert self.cache._depcache.BrokenCount == 0
-            assert self.cache._depcache.DelCount == 0
-        else:
-            # check if it can be installed savely
-            apt_error = False
-            try:
-                self.cache[pkg].markInstall(autoFix=True)
-            except SystemError:
+            if not self.addChannel(item):
+                return False
+            self.last_toggle = item.name
+            self.restoreState()
+        return True
+
+    def tryRemove(self, item):
+        #print "tryRemove", item.pkgname
+        self.cache.clean()
+        pkg = item.pkgname
+        # check if removal can be done
+        self.cache[pkg].markDelete(autoFix=False)
+        if self.cache._depcache.BrokenCount > 0:
+            d = gtk.MessageDialog(parent=self.window_main,
+                                  flags=gtk.DIALOG_MODAL,
+                                  type=gtk.MESSAGE_ERROR,
+                                buttons=gtk.BUTTONS_CLOSE)
+            d.set_title("")
+            d.set_markup("<big><b>%s</b></big>\n\n%s" % \
+                         ((_("Cannot remove '%s'") % pkg),
+                        (_("One or more applications depend on %s. "
+                           "To remove %s and the dependent applications, "
+                           "use the Synaptic package manager.") % (pkg, 
+                                                                   pkg))))
+            d.realize()
+            d.window.set_functions(gtk.gdk.FUNC_MOVE)
+            d.run()
+            d.destroy()
+            self.cache.clean()
+            return False
+        self.cache.clean()
+        return True
+
+    def tryInstall(self, item):
+        #print "tryInstall", item.pkgname
+        pkg = item.pkgname
+        self.cache.clean()
+        # check if it can be installed savely
+        apt_error = False
+
+        # get a action group to speed up the calculation
+        group = apt_pkg.GetPkgActionGroup(self.cache._depcache)
+        # reapply the state of installs
+        (to_add, to_rm) = self.menu.getChanges()
+        for app in to_add:
+            self.cache[app.pkgname].markInstall()
+        del group
+
+        # now install the new pkg
+        try:
+            self.cache[pkg].markInstall(autoFix=True)
+        except SystemError, e:
+            apt_error = True
+        except KeyError:
+            self.error_not_available(item)
+            return False
+        # first check if the package savely replaces a
+        # existing one
+        if (self.cache._depcache.DelCount > 0 and
+            self.cache._depcache.BrokenCount == 0 and
+            not apt_error):
+            if not hasattr(item, "replaces"):
+                return self.canNotInstallApp(pkg)
+            item_replaces = set(item.replaces)
+            removals = set([pkg.name for pkg in self.cache.getChanges() if pkg.markedDelete])
+            if item_replaces != removals:
+                del group
+                return self.canNotInstallApp(pkg)
+            else:
+                # mark the save replace here
+                for r in item_replaces:
+                    apps = self.menu.pkg_to_app[r]
+                    for app in apps:
+                        app.toInstall = False
+                self.treeview_packages.queue_draw()
+
+        # check if we do not conflict with something
+        for app in to_add:
+            if not self.cache[app.pkgname].markedInstall:
                 apt_error = True
-            if self.cache._depcache.BrokenCount > 0 or \
-               self.cache._depcache.DelCount > 0 or apt_error:
-                # FIXME: Resolve conflicts
-                d = gtk.MessageDialog(parent=self.window_main,
-                                      flags=gtk.DIALOG_MODAL,
-                                      type=gtk.MESSAGE_ERROR,
-                                      buttons=gtk.BUTTONS_CLOSE)
-                d.set_title("")
-                d.set_markup("<big><b>%s</b></big>\n\n%s" % (
-                             (_("Cannot install '%s'") % pkg),
-                             (_("This application conflicts with other "
-                                "installed software. To install '%s' "
-                                "the conflicting software "
-                                "must be removed before.\n\n"
-                                "Switch to the advanced mode to resolve this "
-                                "conflict.") % pkg)))
-                d.realize()
-                d.window.set_functions(gtk.gdk.FUNC_MOVE)
-                d.run()
-                d.destroy()
-                # reset the cache
-                # FIXME: a "pkgSimulateInstall,remove"  thing would
-                # be nice
-                self.cache.clean()
-                # FIXME: those assert may be a bit too strong,
-                # we may just rebuild the cache if something is
-                # wrong
-                assert self.cache._depcache.BrokenCount == 0
-                assert self.cache._depcache.DelCount == 0
-                return
+        # then check for real errors
+        if apt_error or self.cache._depcache.BrokenCount > 0:
+            return self.canNotInstallApp(pkg)
+        return True
+       
+    def tryKeep(self, item):
+        #print "tryKeep", item.pkgname
+        # see if we have replaces and if so, apply them
+        if hasattr(item, "replaces"):
+            for r in item.replaces:
+                apps = self.menu.pkg_to_app[r]
+                for app in apps:
+                    app.toInstall = not app.toInstall
+            self.treeview_packages.queue_draw()
+        return True
+
+    def _ensureUnsupportedOrLegalWarning(self, item):
+        """ warn:
+            - on potential legal problems (codecs) always
+            - when a universe or multiverse package is installed for
+              the first time
+        """
+        # check patentBadness
+        if item.patentBadness == True:
+            dia = gtk.MessageDialog(parent=self.window_main,
+                                    type=gtk.MESSAGE_WARNING,
+                                    buttons=gtk.BUTTONS_OK_CANCEL)
+            dia.set_markup("<big><b>%s</b></big>\n\n%s" %
+                           (_("Restricted Software"),
+                            _("The use of some of this software may be "
+                              "restricted in some countries.  You "
+                              "must verify that one of the following is true:\n\n"
+                              "1. These restrictions do not apply in your country of legal residence\n"
+                              "2. You have permission to use this software (for example, a patent license)\n"
+                              "3. You are using this software for research purposes only\n")))
+            res = dia.run()
+            dia.hide()
+            if res != gtk.RESPONSE_OK:
+                return False
+            return True
+
+        # we run the dialog with need_internet = False because we come
+        # here always after _ensureInArchive() was run and that will
+        # enable the sources.list bits for us
+        if item.component == "universe" and not self.seen_universe:
+            dia = DialogUnavailable(self.datadir, self.window_main, item, need_internet=False)
+            res = dia.run()
+            dia.hide()
+            if res != gtk.RESPONSE_OK:
+                return False
+            self.seen_universe = True
+            self.config.set_int("/apps/gnome-app-install/universe_warning", True)
+        if item.component == "multiverse" and not self.seen_multiverse:
+            dia = DialogUnavailable(self.datadir, self.window_main, item, need_internet=False)
+            res = dia.run()
+            dia.hide()
+            if res != gtk.RESPONSE_OK:
+                return False
+            self.seen_multiverse = True
+            self.config.set_int("/apps/gnome-app-install/multiverse_warning", True)
+        return True
+
+    # install toggle on the treeview
+    def on_install_toggle(self, renderer, path):
+        #print "on_install_toggle: %s %s" % (renderer, path)
+        self.setBusy(True)
+        model = self.treeview_packages.get_model()
+        (name, item, popcon) = model[path]
+        pkg = item.pkgname
+
+        # see what should be done
+        want_install = not item.toInstall
+        is_installed = self.cache.has_key(pkg) and self.cache[pkg].isInstalled
+        
+        install = want_install and not is_installed
+        remove =  not want_install and is_installed
+        keep = (want_install and is_installed or
+                not want_install and not is_installed)
+        assert(install ^ remove ^ keep)
+
+        # give a first time warning on universe/multiverse package
+        # installs (because we enable unverse/multiverse by default now)
+        if install:
+            if not self._ensureInArchive(item):
+                self.setBusy(False)
+                return False
+            if not self._ensureUnsupportedOrLegalWarning(item):
+                self.setBusy(False)
+                return False
+
+        # see if it can be done without breaking the cache
+        if install and not self.tryInstall(item):
+            self.setBusy(False)
+            return False
+        elif remove and not self.tryRemove(item):
+            self.setBusy(False)
+            return False
+        elif keep and not self.tryKeep(item):
+            self.setBusy(False)
+            return False
+            
         # the status of the selected package
         status = item.toInstall
         # check if the package provides multiple desktop applications
@@ -486,14 +704,15 @@ class AppInstall(SimpleGladeApp):
                 dia.hide()
                 self.multiple_pkgs_seen.add(item.pkgname)
                 if rt != gtk.RESPONSE_OK:
+                    self.setBusy(False)
                     return
             for app in apps:
                  app.toInstall = not status
         else:
             # invert the current selection
             item.toInstall = not status
-
         self.button_apply.set_sensitive(self.menu.isChanged())
+        self.setBusy(False)
 
     def addChannel(self, item):
         """Ask for confirmation to add the missing channel or
@@ -509,6 +728,12 @@ class AppInstall(SimpleGladeApp):
             return False
         # let go
         if item.component:
+            if item.component == "universe":
+                self.seen_universe = True
+                self.config.set_int("/apps/gnome-app-install/universe_warning", True)
+            if item.component == "multiverse":
+                self.seen_multiverse = True
+                self.config.set_int("/apps/gnome-app-install/multiverse_warning", True)
             self.enableComponent(item.component)
             # FIXME: make sure to fix this after release in a more elegant
             #        and generic way
@@ -539,7 +764,10 @@ class AppInstall(SimpleGladeApp):
             of the corresponding application
             """
             (name, item, popcon) = model[iter]
-            rank = int((5 * item.popcon / self.menu.popcon_max))
+            #rank = int((5 * item.popcon / self.menu.popcon_max))
+            rank = 0
+            if item.popcon > 0:
+                rank = 5*log(item.popcon)/log(self.menu.popcon_max)
             pix_rating = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True,
                                         8, 96, 16) # depth, width, height
             pix_rating.fill(0x0)
@@ -555,11 +783,12 @@ class AppInstall(SimpleGladeApp):
 
         def package_view_func(cell_layout, renderer, model, iter):
             app = model.get_value(iter, COL_ITEM)
-            name = app.name
-            desc = app.description
-            current = app.isInstalled
+            name = xmlescape(app.name)
+            desc = xmlescape(app.description)
             future = app.toInstall
-            available = app.available
+            current = self.menu.itemIsInstalled(app)
+            available = self.menu.itemAvailable(app)
+
             if current != future:
                 markup = "<b>%s</b>\n<small><b>%s</b></small>" % (name, desc)
             else:
@@ -619,7 +848,7 @@ class AppInstall(SimpleGladeApp):
         renderer_app = gtk.CellRendererText()
         renderer_app.set_property("ellipsize", pango.ELLIPSIZE_END)
         column_app.pack_start(renderer_app, True)
-        column_app.add_attribute(renderer_app, "markup", COL_NAME)
+        #column_app.add_attribute(renderer_app, "markup", COL_NAME)
         column_app.set_cell_data_func (renderer_app, package_view_func)
 
         self.treeview_packages.append_column(column_app_status)
@@ -659,12 +888,13 @@ class AppInstall(SimpleGladeApp):
     def restoreState(self):
         """ restore the current state of the app """
         # set category
+        self.window_main.set_sensitive(False)
         self.treeview_categories.set_cursor(self.cursor_categories_path)
         model = self.treeview_packages.get_model()
         # reapply search
-        query = self.search_entry.get_text()
-        if query:
-            self.on_search_timeout()
+        #query = self.search_entry.get_text()
+        #if query:
+        #    self.on_search_timeout()
         # remark all packages that were marked for installation
         for item in self.to_add:
             if self.cache.has_key(item.pkgname):
@@ -687,9 +917,9 @@ class AppInstall(SimpleGladeApp):
                 apps = self.menu.pkg_to_app[item.pkgname]
                 for app in apps:
                     app.toInstall = item.toInstall
-        # redraw the treeview so that all check buttons are updated
-        self.treeview_packages.queue_draw()
-
+        # restore search
+        if len(self.search_entry.get_text()) > 0:
+            self.on_search_timeout()
         # find package
         for it in iterate_list_store(model,model.get_iter_first()):
             name = model.get_value(it, COL_NAME)
@@ -705,21 +935,35 @@ class AppInstall(SimpleGladeApp):
                 path = model.get_path(it)
                 self.treeview_packages.set_cursor(path)
                 break
-
+        self.window_main.set_sensitive(True)
+        # redraw the treeview so that all check buttons are updated
+        self.treeview_packages.queue_draw()
+        
     def updateCache(self, filter=SHOW_ONLY_MAIN):
         self.window_main.set_sensitive(False)
         self.setBusy(True)
-            
-        progress = GtkOpProgressWindow(self.glade,self.window_main)
+
+        # honor transient_for cmdline arguemnt
+        self.window_main.realize()
+        progress_transient_for = self.window_main
+        if self.transient_for is not None:
+            progress_transient_for = self.transient_for
+        progress = GtkOpProgressWindow(self.glade, progress_transient_for)
         try:
-            self.cache = MyCache(progress)
+            if not hasattr(self, "cache"):
+                self.cache = MyCache(progress)
+            else:
+                self.cache.open(progress)
         except Exception, e:
             # show an error dialog if something went wrong with the cache
             header = _("Failed to check for installed and available applications")
-            msg = _("This is a major failure of your software " \
-                    "management system. Check the file permissions and "\
-                    "correctness of the file '/etc/apt/sources.list' and "\
-                    "reload the software information: 'sudo apt-get update'.")
+            msg = _("This is a major failure of your software " 
+                    "management system. Please check for broken packages "
+                    "with synaptic, check the file permissions and "
+                    "correctness of the file '/etc/apt/sources.list' and "
+                    "reload the software information with: "
+                    "'sudo apt-get update' and 'sudo apt-get install -f'."
+                    )
             print e
             d = gtk.MessageDialog(parent=self.window_main,
                                   flags=gtk.DIALOG_MODAL,
@@ -733,13 +977,17 @@ class AppInstall(SimpleGladeApp):
             d.destroy()
             sys.exit(1)
 
-        self.menu = ApplicationMenu(self.desktopdir,
-                                    self.cachedir,
-                                    self.cache,
-                                    self.treeview_categories,
-                                    self.treeview_packages,
-                                    progress, filter)
-
+        if not hasattr(self, "menu"):
+            self.menu = ApplicationMenu(self.desktopdir,
+                                        self.cachedir,
+                                        self.cache,
+                                        self.treeview_categories,
+                                        self.treeview_packages,
+                                        progress, filter,
+                                        activation_style=self.activation_style)
+        else:
+            self.menu.refreshAfterCacheChange(progress)
+            
         # move to "All" category per default
         self.treeview_categories.set_cursor((0,))
 
@@ -778,17 +1026,27 @@ class AppInstall(SimpleGladeApp):
     # ----------------------------
     # Main window button callbacks
     # ----------------------------
-    
+
+    def on_button_preferences_clicked(self, widget):
+        subprocess.Popen(["/usr/bin/gksu", "software-properties-gtk"])
+
     def on_button_help_clicked(self, widget):
         subprocess.Popen(["/usr/bin/yelp", "ghelp:gnome-app-install"])
 
     def applyChanges(self, final=False):
         #print "do_apply()"
+	self.activation_style.userApprovedNotify()
         (to_add, to_rm) = self.menu.getChanges()
         # Set a busy cursor
         self.setBusy(True)
         # Get the selections delta for the changes and apply them
-        ret = self.packageWorker.perform_action(self.window_main, to_add, to_rm)
+        pkgs_add = set([item.pkgname for item in to_add])
+        pkgs_rm = set([item.pkgname for item in to_rm])
+        if len(pkgs_rm) > 0:
+            additional_rm = self.cache.getDependantAutoDeps(pkgs_rm)
+            pkgs_rm |= additional_rm
+        ret = self.packageWorker.perform_action(self.window_main, self.cache,
+                                                pkgs_add, pkgs_rm)
 
         # error from gksu
         if ret != 0:
@@ -811,6 +1069,7 @@ class AppInstall(SimpleGladeApp):
         
         # And reset the cursor
         self.setBusy(False)
+	self.activation_style.changesSuccessfulNotify()
         return True
 
     def on_button_ok_clicked(self, button):
@@ -821,7 +1080,6 @@ class AppInstall(SimpleGladeApp):
         # succesfully been applied (otherwise cancel)
         if self.confirmChanges():
             if self.applyChanges(final=True):
-	        if self.menu.mimeSearch: self.menu.mimeSearch.retry_open()
                 self.quit()
 
     def confirmChanges(self):
@@ -847,6 +1105,7 @@ class AppInstall(SimpleGladeApp):
         ret = self.confirmChanges()
         if ret == True :
             self.applyChanges()
+        
 
     def on_search_timeout(self):
         """
@@ -881,7 +1140,8 @@ class AppInstall(SimpleGladeApp):
         """
         if self.search_timeout_id > 0:
             gobject.source_remove(self.search_timeout_id)
-        self.search_timeout_id = gobject.timeout_add(500,self.on_search_timeout)
+        timeout = self.config.get_int("/apps/gnome-app-install/search-timeout")
+        self.search_timeout_id = gobject.timeout_add(timeout, self.on_search_timeout)
 
     def on_button_clear_clicked(self, button):
         self.search_entry.set_text("")
@@ -907,9 +1167,12 @@ class AppInstall(SimpleGladeApp):
                 
     def reloadSources(self):
         self.window_main.set_sensitive(False)
+        self.saveState()
         ret = self.packageWorker.perform_action(self.window_main,
+                                                None,
                                                 action=PackageWorker.UPDATE)
         self.updateCache(filter=self.menu.filter)
+        self.restoreState()
         self.window_main.set_sensitive(True)
         return ret
 
@@ -937,15 +1200,13 @@ class AppInstall(SimpleGladeApp):
                    "--",
                    "apt-key", "add",channelkey]
             subprocess.call(cmd)
-                
+
     def enableComponent(self, component):
         """ Enables a component of the current distribution
             (in a seperate file in /etc/apt/sources.list.d/$dist-$comp)
         """
         # get the codename of the currently used distro
-        pipe = os.popen("lsb_release -c -s")
-        distro = pipe.read().strip()
-        del pipe
+        distro = self.distro
         # sanity check
         if component == "":
             print "no repo found in enableRepository"
@@ -1030,8 +1291,9 @@ class AppInstall(SimpleGladeApp):
         self.quit()
             
     def quit(self):
-        gtk.main_quit()
-        sys.exit(0)
+        #gtk.main_quit()
+	self.activation_style.quitHook() # may exit
+	sys.exit(0)
     
     def show_description(self, item):
         """Collect and show some information about the package that 
@@ -1054,7 +1316,7 @@ class AppInstall(SimpleGladeApp):
                             icons.append([desktop_environment_mapping[dependencies][1], desktop_environment_mapping[dependencies][0] % item.name])
                         break
 
-        if item.available:
+        if self.menu.itemAvailable(item):
             pkg = self.cache[item.pkgname]
             rough_desc = pkg.description.rstrip(" \n\t")
             # the first line is the short description
